import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter_map/flutter_map.dart';
import 'package:flutter_map_location_marker/flutter_map_location_marker.dart';
import 'package:flutter_svg/svg.dart';
import 'package:furman_now/src/utils/theme.dart';
import 'package:furman_now/src/widgets/map/filter_chip.dart';
import 'package:furman_now/src/widgets/map/rotate_compass.dart';
import 'package:latlong2/latlong.dart';

class MapScreen extends StatefulWidget {
  const MapScreen({Key? key}) : super(key: key);

  @override
  State<MapScreen> createState() => _MapScreenState();
}

class _MapScreenState extends State<MapScreen>
    with SingleTickerProviderStateMixin {
  final MapController _mapController = MapController();

  late final AnimationController _animationController = AnimationController(
    duration: const Duration(milliseconds: 300),
    vsync: this,
  );

  late CenterOnLocationUpdate _centerOnLocationUpdate;
  late StreamController<double?> _centerCurrentLocationStreamController;

  var _rotation = 0.0;

  @override
  void initState() {
    super.initState();
    _mapController.mapEventStream.listen((event) {
      if (event is MapEventRotate) {
        setState(() {
          _rotation = _mapController.rotation * (2 * pi) / 360;
        });
      }
    });
    _centerOnLocationUpdate = CenterOnLocationUpdate.always;
    _centerCurrentLocationStreamController = StreamController<double?>();
  }

  @override
  void dispose() {
    super.dispose();
    _animationController.dispose();
  }

  void resetRotation() async {
    // take the shortest rotation path
    var end = _mapController.rotation > 180 ? 360.0 : 0.0;
    var animation = Tween<double>(
      begin: _mapController.rotation,
      end: end,
    ).animate(CurvedAnimation(
      parent: _animationController,
      curve: Curves.easeInOut,
    ));

    animationListener() {
      _mapController.rotate(animation.value);
    }

    animation.addListener(animationListener);

    await _animationController.forward();

    animation.removeListener(animationListener);
    _animationController.reset();

    _mapController.rotate(0);
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: Container(
        color: const Color(0xffb7acc9),
        child: SafeArea(
          top: false,
          child: Stack(
            children: [
              FlutterMap(
                mapController: _mapController,
                options: MapOptions(
                  center: LatLng(34.925926, -82.439397),
                  enableMultiFingerGestureRace: true,
                  rotationWinGestures: MultiFingerGesture.all,
                  pinchZoomThreshold: 0.2,
                  rotationThreshold: 8,
                  zoom: 15,
                  minZoom: 12,
                  maxZoom: 18,
                  plugins: [
                    LocationMarkerPlugin(
                      centerCurrentLocationStream:
                        _centerCurrentLocationStreamController.stream,
                      centerOnLocationUpdate: _centerOnLocationUpdate,
                    ),
                  ],
                  onPositionChanged: (MapPosition position, bool hasGesture) {
                    if (hasGesture) {
                      setState(
                        () => _centerOnLocationUpdate = CenterOnLocationUpdate.never,
                      );
                    }
                  },
                ),
                layers: [
                  TileLayerOptions(
                    urlTemplate:
                        "https://tile.openstreetmap.org/{z}/{x}/{y}.png",
                    userAgentPackageName: 'edu.furman.now',
                  ),
                  LocationMarkerLayerOptions(),
                ],
                nonRotatedChildren: [
                  AttributionWidget(
                    attributionBuilder: (BuildContext context) {
                      return const ColoredBox(
                        color: Color(0xCCFFFFFF),
                        child: Padding(
                          padding: EdgeInsets.all(3),
                          child: Text("©️ OpenStreetMap contributors"),
                        ),
                      );
                    },
                  ),
                ],
              ),
              // Rotation reset fab
              Positioned(
                top: 12,
                left: 0,
                right: 0,
                child: SafeArea(
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Padding(
                        padding: const EdgeInsets.symmetric(horizontal: 12),
                        child: Container(
                          width: double.infinity,
                          height: 50,
                          padding: const EdgeInsets.only(left: 10, right: 20),
                          decoration: const BoxDecoration(
                            color: Colors.white,
                            borderRadius: BorderRadius.all(Radius.circular(60)),
                            boxShadow: [
                              BoxShadow(
                                color: Color(0x33000000),
                                blurRadius: 8,
                              ),
                            ],
                          ),
                          child: Stack(
                            children: [
                              Align(
                                alignment: Alignment.centerLeft,
                                child: Wrap(
                                  crossAxisAlignment: WrapCrossAlignment.center,
                                  children: [
                                    SvgPicture.asset("assets/images/bell-tower.svg", color: Theme.of(context).primaryColor, height: 32),
                                    const SizedBox(width: 10),
                                    Text(
                                      "Search locations",
                                      style: furmanTextStyle(TextStyle(
                                        fontSize: 18,
                                        fontWeight: FontWeight.w500,
                                        color: Colors.grey.shade500,
                                      )),
                                    ),
                                  ],
                                ),
                              ),
                            ],
                          ),
                        ),
                      ),
                      // const SizedBox(height: 12),
                      SingleChildScrollView(
                        padding: const EdgeInsets.symmetric(horizontal: 12, vertical: 12),
                        scrollDirection: Axis.horizontal,
                        child: Wrap(
                          spacing: 6,
                          children: const [
                            MapFilterChip(icon: Icons.restaurant, text: "Restaurants"),
                            MapFilterChip(icon: Icons.train, text: "Transportation"),
                            MapFilterChip(icon: Icons.school, text: "Campus Buildings"),
                          ],
                        ),
                      ),
                      MapRotateCompass(rotation: _rotation, resetRotation: resetRotation),
                    ],
                  ),
                ),
              ),
              Positioned(
                right: 20,
                bottom: 20,
                child: FloatingActionButton(
                  onPressed: () {
                    // Automatically center the location marker on the map when location updated until user interact with the map.
                    setState(
                      () => _centerOnLocationUpdate = CenterOnLocationUpdate.always,
                    );
                    // Center the location marker on the map and zoom the map to level 18.
                    _centerCurrentLocationStreamController.add(16);
                  },
                  child: const Icon(
                    Icons.my_location,
                    color: Colors.white,
                  ),
                )
              )
            ],
          ),
        ),
      ),
    );
  }
}
