import 'dart:convert';

import 'package:furman_now/src/services/events/event.dart';
import 'package:http/http.dart' as http;

class EventsService {
  static Future<List<Event>> fetchEvents() async {
    var athleticsEvents = await fetchAthleticsEvents();
    var clpEvents = await fetchClpEvents();
    List<Event> eventsList = List.from(athleticsEvents)..addAll(clpEvents);
    eventsList.sort((Event a, Event b) => a.time.compareTo(b.time));
    return eventsList;
  }

  static Future<Iterable<AthleticsEvent>> fetchAthleticsEvents() async {
    final response = await http
        .get(Uri.parse("https://cs.furman.edu/~csdaemon/FUNow/athleticsGet.php"));

    if (response.statusCode == 200) {
      // If the server did return a 200 OK response,
      // then parse the JSON.
      final eventsJson = jsonDecode(response.body);
      return (eventsJson["results"] as List<dynamic>).map((event) =>
        AthleticsEvent.fromJson(event)
      );
    } else {
      // If the server did not return a 200 OK response,
      // then throw an exception.
      throw Exception('Failed to load athletics events.');
    }
  }

  static Future<Iterable<ClpEvent>> fetchClpEvents() async {
    final response = await http
        .get(Uri.parse("https://cs.furman.edu/~csdaemon/FUNow/clpGet.php"));

    if (response.statusCode == 200) {
      // If the server did return a 200 OK response,
      // then parse the JSON.
      final eventsJson = jsonDecode(response.body);
      return (eventsJson["results"] as List<dynamic>).map((event) =>
          ClpEvent.fromJson(event)
      );
    } else {
      // If the server did not return a 200 OK response,
      // then throw an exception.
      throw Exception('Failed to load athletics events.');
    }
  }
}

class AthleticsEvent implements Event {
  @override
  final String title;
  @override
  final DateTime time;
  @override
  final String location;
  @override
  final String category = "Athletics";

  final String sportTitle;
  final String sportShort;
  final AthleticsEventLocation locationIndicator;
  final String opponent;

  AthleticsEvent._({
    required this.title,
    required this.time,
    required this.location,

    required this.sportTitle,
    required this.sportShort,
    required this.locationIndicator,
    required this.opponent,
  });

  factory AthleticsEvent({
    required time,
    required location,
    required sportTitle,
    required sportShort,
    required locationIndicator,
    required opponent,
  }) {
    // Determine AthleticsEventLocation from string
    AthleticsEventLocation eventLocationIndicator;
    switch(locationIndicator) {
      case ("H"):
        eventLocationIndicator = AthleticsEventLocation.home;
        break;
      case ("A"):
        eventLocationIndicator = AthleticsEventLocation.away;
        break;
      case ("N"):
      default:
        eventLocationIndicator = AthleticsEventLocation.neither;
        break;
    }

    // Generate event title
    var eventTitleSeparator =
      eventLocationIndicator == AthleticsEventLocation.away
          ? "at"
          : "vs";

    return AthleticsEvent._(
      title: '$sportTitle $eventTitleSeparator $opponent',
      time: time,
      location: location,
      sportTitle: sportTitle,
      sportShort: sportShort,
      locationIndicator: eventLocationIndicator,
      opponent: opponent,
    );
  }

  factory AthleticsEvent.fromJson(Map<String, dynamic> json) {
    return AthleticsEvent(
      sportTitle: json["sportTitle"],
      sportShort: json["sportShort"],
      time: DateTime.parse(json["eventdate"]),
      location: json["location"],
      locationIndicator: json["location_indicator"],
      opponent: json["opponent"],
    );
  }
}

enum AthleticsEventLocation {
  home,
  away,
  neither,
}

class ClpEvent implements Event {
  @override
  final String title;
  @override
  DateTime get time {
    return startTime;
  }
  @override
  final String location;
  @override
  final String category;

  final DateTime startTime;
  final DateTime endTime;
  final String organization;
  final String description;

  ClpEvent({
    required this.title,
    required this.startTime,
    required this.endTime,
    required this.location,
    required this.category,
    required this.organization,
    required this.description,
  });

  factory ClpEvent.fromJson(Map<String, dynamic> json) {
    return ClpEvent(
      title: json["title"],
      startTime: DateTime.parse(json["date"] + " " + json["start"]),
      endTime: DateTime.parse(json["date"] + " " + json["end"]),
      location: json["location"],
      category: json["eventType"],
      organization: json["organization"],
      description: json["description"],
    );
  }
}