import 'dart:convert';
import 'package:crypto/crypto.dart';
import 'dart:typed_data';
import 'package:convert/convert.dart';
import 'package:furman_now/secrets.dart';

class BarcodeService {
  static generateGetBarcode({ int? timestamp }) {
    // seconds since epoch
    timestamp ??= DateTime.now().millisecondsSinceEpoch ~/ 1000;

    // final timestamp = DateTime.now().millisecondsSinceEpoch ~/ 1000;
    // const timestamp = 1661567550;

    final timestampBytes = _int64BigEndianBytes(timestamp);

    const cbordKey = Secrets.cbordKey;
    const institutionKey = Secrets.institutionKey;
    const patronKey = Secrets.testPatronKey;

    final cbordKeyBytes = Uint8List.fromList(hex.decode(utf8.decode(base64Decode(cbordKey))));
    final institutionKeyBytes = Uint8List.fromList(hex.decode(institutionKey));
    final patronKeyBytes = _int32LittleEndianBytes(patronKey);
    final sharedKeyBytes = _xorEncrypt(cbordKeyBytes, institutionKeyBytes);

    final checksumDigit = _luhnGenerate(patronKey.toString());

    // print(hex.encode(_int32BigEndianBytes(timestamp)));
    final barcodeOtc = int.parse(_hotp(sharedKeyBytes, timestampBytes, 9, hash: sha256));
    final barcodeOtcBytes = _int32LittleEndianBytes(barcodeOtc);

    final encryptedBytes = _xorEncrypt(patronKeyBytes, barcodeOtcBytes);
    final encrypted = ByteData.view(encryptedBytes.buffer).getInt32(0, Endian.little);

    final barcodeData = "${timestamp.toString().padLeft(10, "0")}1${encrypted.toString().padLeft(10, "0")}$checksumDigit";

    return barcodeData;
  }

  static int _luhnGenerate(String code) {
    const computed = [0, 2, 4, 6, 8, 1, 3, 5, 7, 9];
    var sum = 0,
        digit = 0,
        i = code.length,
        even = false;

    while (i-- > 0) {
      digit = int.parse(code[i]);
      sum += (even = !even) ? computed[digit] : digit;
    }

    return (sum * 9) % 10;
  }

  static Uint8List _xorEncrypt(List<int> subject, List<int> key) {
    var out = Uint8List(subject.length);
    for (int i = 0; i < subject.length; i++) {
      out[i] = subject[i] ^ key[i % key.length];
    }
    return out;
  }

  static _hotp(Uint8List key, Uint8List counter, int digits, { Hash hash = sha256 }) {
    final mac = Uint8List.fromList(Hmac(hash, key).convert(counter).bytes);
    final offset = mac[mac.length - 1] & 0x0f;
    final binary = ByteData.view(mac.buffer).getInt32(offset, Endian.big) & 0x7fffffff;
    var binaryString = binary.toString();
    if (binaryString.length - digits > 0) {
      binaryString = binaryString.substring(binaryString.length - digits);
    }
    binaryString = binaryString.padLeft(digits, "0");
    return binaryString;
  }

  static Uint8List _int32LittleEndianBytes(int value) =>
      Uint8List(4)..buffer.asByteData().setInt32(0, value, Endian.little);

  static Uint8List _int64BigEndianBytes(int value) {
    var sendValueBytes = ByteData(8);
    // setUint64 not implemented on some systems so use setUint32 in
    // those cases. Leading zeros to pad to equal 64 bit.
    // Epoch as 32-bit good until 2038 Jan 19 @ 03:14:07
    try {
      sendValueBytes.setUint64(0, value, Endian.big);
    } on UnsupportedError {
      sendValueBytes.setUint32(0, value, Endian.big);
    }
    return sendValueBytes.buffer.asUint8List();
  }
}
