import 'dart:convert';

import 'package:flutter/painting.dart';
import 'package:latlong2/latlong.dart';
import 'package:http/http.dart' as http;

class RestaurantService {
  static Future<List<Restaurant>> fetchRestaurants() async {
    var restaurants = (await _fetchRestaurants()).toList();
    await _fetchRestaurantHours(restaurants);
    return restaurants;
  }

  static Future<Iterable<Restaurant>> _fetchRestaurants() async {
    final response = await http
        .get(Uri.parse("https://cs.furman.edu/~csdaemon/FUNow/restaurantGet.php"));

    if (response.statusCode == 200) {
      // If the server did return a 200 OK response,
      // then parse the JSON.
      final restaurantsJson = jsonDecode(response.body);
      return (restaurantsJson["results"] as List<dynamic>).map((restaurant) =>
          Restaurant.fromJson(restaurant)
      );
    } else {
      // If the server did not return a 200 OK response,
      // then throw an exception.
      throw Exception('Failed to load athletics events.');
    }
  }

  static Future<void> _fetchRestaurantHours(List<Restaurant> restaurants) async {
    final response = await http
        .get(Uri.parse("https://cs.furman.edu/~csdaemon/FUNow/restaurantHoursGet.php"));

    if (response.statusCode == 200) {
      // If the server did return a 200 OK response,
      // then parse the JSON.
      final hoursListJson = jsonDecode(response.body);
      for (var hoursJson in (hoursListJson["results"] as List<dynamic>)) {
        var hours = Hours.fromJson(hoursJson);
        restaurants.firstWhere((restaurant) => restaurant.id == hours.id).hoursList.add(hours);
      }
    } else {
      // If the server did not return a 200 OK response,
      // then throw an exception.
      throw Exception('Failed to load athletics events.');
    }
  }
}

class Restaurant {
  int id;
  String name;
  String? shortName;
  String location;
  LatLng mapLocation;
  int frequency;
  int busyness;
  String? url;
  List<Hours> hoursList = <Hours>[];

  Restaurant({
    required this.id,
    required this.name,
    this.shortName,
    required this.location,
    required this.mapLocation,
    required this.frequency,
    required this.busyness,
    this.url,
  });

  bool get isOpen {
    for(var hours in hoursList) {
      var now = DateTime.now();
      var today = Weekday.values[now.weekday - 1];
      if (hours.daysOfWeek.days[today] == true) {
        var currentTime = Duration(hours: now.hour, minutes: now.minute, seconds: now.second);
        if(hours.startTime != null && hours.endTime != null) {
          if (hours.startTime! < currentTime && hours.endTime! > currentTime) {
            return true;
          }
        }
      }
    }
    return false;
  }

  // TODO: this is the absolute worst
  String get imageUri {
    return "https://cs.furman.edu/~csdaemon/FUNow/appIcons/$name Icon.png";
  }

  ImageProvider get image {
    return NetworkImage(imageUri);
  }

  factory Restaurant.fromJson(Map<String, dynamic> json) {
    return Restaurant(
        id: int.parse(json["id"]),
        name: json["fullname"],
        shortName: json["name"],
        location: json["location"],
        mapLocation: LatLng(double.parse(json["latitude"]), double.parse(json["longitude"])),
        frequency: int.parse(json["frequency"]),
        busyness: int.parse(json["busyness"]),
        url: (json["url"] != null && (json["url"] as String).isNotEmpty) ? json["url"] : null,
    );
  }
}

class Hours {
  int id;
  String? meal;
  Duration? startTime;
  Duration? endTime;
  DaysOfWeek daysOfWeek;
  int dayOrder;

  Hours({
    required this.id,
    this.meal,
    this.startTime,
    this.endTime,
    required this.daysOfWeek,
    required this.dayOrder,
  });

  static Duration _getDurationFromString(String s) {
    var timeComponents = s.split(":");
    var hours = int.parse(timeComponents[0]);
    var minutes = int.parse(timeComponents[1]);
    var seconds = int.parse(timeComponents[2]);

    return Duration(hours: hours, minutes: minutes, seconds: seconds);
  }

  factory Hours.fromJson(Map<String, dynamic> json) {
    return Hours(
      id: int.parse(json["id"]),
      meal: json["meal"],
      startTime: (json["start"] != null) ? _getDurationFromString(json["start"]) : null,
      endTime: (json["end"] != null) ? _getDurationFromString(json["end"]) : null,
      daysOfWeek: DaysOfWeek.parse(json["dayOfWeek"]),
      dayOrder: int.parse(json["dayOrder"]),
    );
  }
}

class DaysOfWeek {
  final Weekday startDay;
  final Weekday endDay;
  final Map<Weekday, bool> days;

  DaysOfWeek({
    required this.startDay,
    required this.endDay,
    required this.days,
  });

  static Weekday _getDayFromShortName(String shortName) {
    switch(shortName.toLowerCase()) {
      case "mon":
        return Weekday.monday;
      case "tue":
        return Weekday.tuesday;
      case "wed":
        return Weekday.wednesday;
      case "thu":
        return Weekday.thursday;
      case "fri":
        return Weekday.friday;
      case "sat":
        return Weekday.saturday;
      case "sun":
        return Weekday.sunday;
      default:
        throw "Invalid date short name.";
    }
  }

  factory DaysOfWeek.parse(String s) {
    final Map<Weekday, bool> days = { for (var e in Weekday.values) e : false };

    // single day
    if (!s.contains("-")) {
      var day = _getDayFromShortName(s);
      days.update(day, (val) => true);
      return DaysOfWeek(startDay: day, endDay: day, days: days);
    }

    // date range
    var dayNames = s.split("-");
    final startDay = _getDayFromShortName(dayNames[0]);
    final endDay = _getDayFromShortName(dayNames[1]);
    int i = startDay.index;

    // loop through to add all dates in date range to list
    while(i != endDay.index) {
      days.update(Weekday.values[i], (val) => true);

      if (i < Weekday.values.length - 1) {
        i++;
      } else {
        i = 0;
      }
    }
    days.update(endDay, (val) => true);

    return DaysOfWeek(
      startDay: startDay,
      endDay: endDay,
      days: days,
    );
  }
}

enum Weekday {
  monday,
  tuesday,
  wednesday,
  thursday,
  friday,
  saturday,
  sunday,
}
